@@ -1,9 +1,10 @@
 #' Run the same \pkg{brms} model on multiple datasets
 #' 
-#' Run the same \pkg{brms} model on multiple datasets and
-#' then combine the results into one fitted model object. 
-#' This is useful in particular for multiple missing value imputation,
-#' where the same model is fitted on multiple imputed data sets.
+#' Run the same \pkg{brms} model on multiple datasets and then combine the
+#' results into one fitted model object. This is useful in particular for
+#' multiple missing value imputation, where the same model is fitted on multiple
+#' imputed data sets. Models can be run in parallel using the \pkg{future}
+#' package.
 #' 
 #' @inheritParams brm
 #' @param data A list of data.frames each of which will
@@ -41,6 +42,12 @@
 #' plot(fit_imp2, pars = "^b_")
 #' # investigate convergence of the original models
 #' fit_imp2$rhats
+#' 
+#' # use the future package for parallelization
+#' library(future)
+#' plan(multiprocess)
+#' fit_imp3 <- brm_multiple(bmi~age+hyp+chl, data = imp, chains = 1)
+#' summary(fit_imp3)
 #' }
 #' 
 #' @export
@@ -50,7 +57,7 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
                          sparse = FALSE, knots = NULL, stanvars = NULL,
                          stan_funs = NULL, combine = TRUE, 
                          seed = NA, file = NULL, ...) {
-  
+  require_package("future")
   combine <- as_one_logical(combine)
   if (!is.null(file)) {
     # optionally load saved model object
@@ -74,13 +81,13 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
   } else if (!(is.list(data) && is.vector(data))) {
     stop2("'data' must be a list of data.frames.")
   }
-  fits <- vector("list", length(data))
+  fits <- futures <- vector("list", length(data))
   message("Fitting imputed model 1")
   args <- nlist(
     formula, data = data[[1]], family, prior, autocor, cov_ranef,
     sample_prior, sparse, knots, stanvars, stan_funs, seed, ...
   )
-  fits[[1]] <- do.call(brm, args)
+  fits[[1]] <- run(brm, args)
   fits[[1]]$data.name <- data.name
   rhats <- data.frame(as.list(rhat(fits[[1]])))
   if (any(rhats > 1.1)) {
@@ -88,7 +95,13 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
   }
   for (i in seq_along(data)[-1]) {
     message("Fitting imputed model ", i)
-    fits[[i]] <- update(fits[[1]], newdata = data[[i]], recompile = FALSE, ...)
+    futures[[i]] <- future::future(
+      update(fits[[1]], newdata = data[[i]], recompile = FALSE, ...),
+      packages = "brms"
+    )
+  }
+  for (i in seq_along(data)[-1]) {
+    fits[[i]] <- future::value(futures[[i]]) 
     rhat_i <- data.frame(as.list(rhat(fits[[i]])))
     if (any(rhat_i > 1.1)) {
       warning2("Imputed model ", i, " did not converge.")
