@@ -11,7 +11,7 @@ predict_internal.mvbrmsdraws <- function(draws, ...) {
   } else {
     out <- lapply(draws$resps, predict_internal, ...)
     along <- ifelse(length(out) > 1L, 3, 2)
-    out <- do.call(abind, c(out, along = along))
+    out <- run(abind, c(out, along = along))
   }
   out
 }
@@ -31,12 +31,13 @@ predict_internal.brmsdraws <- function(draws, summary = TRUE, transform = NULL,
   N <- choose_N(draws)
   out <- lapply(seq_len(N), predict_fun, draws = draws, ...)
   if (grepl("_mv$", draws$f$fun)) {
-    out <- do.call(abind, c(out, along = 3))
+    out <- run(abind, c(out, along = 3))
     out <- aperm(out, perm = c(1, 3, 2))
     dimnames(out)[[3]] <- names(draws$resps)
   } else {
-    out <- do.call(cbind, out) 
+    out <- run(cbind, out) 
   }
+  colnames(out) <- NULL
   # percentage of invalid samples for truncated discrete models
   # should always be zero for all other models
   pct_invalid <- get_pct_invalid(out, lb = draws$data$lb, ub = draws$data$ub) 
@@ -49,7 +50,7 @@ predict_internal.brmsdraws <- function(draws, summary = TRUE, transform = NULL,
   out <- reorder_obs(out, draws$old_order, sort = sort)
   # transform predicted response samples before summarizing them 
   if (!is.null(transform)) {
-    out <- do.call(transform, list(out))
+    out <- run(transform, list(out))
   }
   attr(out, "levels") <- draws$data$cats
   if (summary) {
@@ -135,7 +136,7 @@ predict_gaussian_mv <- function(i, draws, ...) {
   .predict <- function(s) {
     rmulti_normal(1, mu = Mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_mv <- function(i, draws, ...) {
@@ -145,7 +146,7 @@ predict_student_mv <- function(i, draws, ...) {
   .predict <- function(s) {
     rmulti_student_t(1, df = nu[s], mu = Mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_cov <- function(i, draws, ...) {
@@ -155,7 +156,7 @@ predict_gaussian_cov <- function(i, draws, ...) {
   .predict <- function(s) {
     rmulti_normal(1, mu = mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_cov <- function(i, draws, ...) {
@@ -166,7 +167,7 @@ predict_student_cov <- function(i, draws, ...) {
   .predict <- function(s) {
     rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_lagsar <- function(i, draws, ...) {
@@ -179,7 +180,7 @@ predict_gaussian_lagsar <- function(i, draws, ...) {
   }
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_lagsar <- function(i, draws, ...) {
@@ -193,7 +194,7 @@ predict_student_lagsar <- function(i, draws, ...) {
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
   nu <- get_dpar(draws, "nu")
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_errorsar <- function(i, draws, ...) {
@@ -205,7 +206,7 @@ predict_gaussian_errorsar <- function(i, draws, ...) {
   }
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_errorsar <- function(i, draws, ...) {
@@ -218,7 +219,7 @@ predict_student_errorsar <- function(i, draws, ...) {
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
   nu <- get_dpar(draws, "nu")
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_fixed <- function(i, draws, ...) {
@@ -227,7 +228,7 @@ predict_gaussian_fixed <- function(i, draws, ...) {
   .predict <- function(s) {
     rmulti_normal(1, mu = mu[s, ], Sigma = draws$ac$V)
   }
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_fixed <- function(i, draws, ...) {
@@ -237,7 +238,7 @@ predict_student_fixed <- function(i, draws, ...) {
   .predict <- function(s) {
     rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = draws$ac$V)
   }
-  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
+  run(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_binomial <- function(i, draws, ntrys = 5, ...) {
@@ -594,17 +595,17 @@ rng_continuous <- function(nrng, dist, args, lb = NULL, ub = NULL) {
   if (is.null(lb) && is.null(ub)) {
     # sample as usual
     rdist <- paste0("r", dist)
-    out <- do.call(rdist, c(nrng, args))
+    out <- run(rdist, c(nrng, args))
   } else {
     # sample from truncated distribution
     if (is.null(lb)) lb <- -Inf
     if (is.null(ub)) ub <- Inf
     pdist <- paste0("p", dist)
     qdist <- paste0("q", dist)
-    plb <- do.call(pdist, c(list(lb), args))
-    pub <- do.call(pdist, c(list(ub), args))
+    plb <- run(pdist, c(list(lb), args))
+    pub <- run(pdist, c(list(ub), args))
     rng <- list(runif(nrng, min = plb, max = pub))
-    out <- do.call(qdist, c(rng, args))
+    out <- run(qdist, c(rng, args))
     # remove infinte values caused by numerical imprecision
     out[out %in% c(-Inf, Inf)] <- NA
   }
@@ -627,12 +628,12 @@ rng_discrete <- function(nrng, dist, args, lb = NULL, ub = NULL, ntrys = 5) {
   rdist <- get(paste0("r", dist), mode = "function")
   if (is.null(lb) && is.null(ub)) {
     # sample as usual
-    do.call(rdist, c(nrng, args))
+    run(rdist, c(nrng, args))
   } else {
     # sample from truncated distribution via rejection sampling
     if (is.null(lb)) lb <- -Inf
     if (is.null(ub)) ub <- Inf
-    rng <- matrix(do.call(rdist, c(nrng * ntrys, args)), ncol = ntrys)
+    rng <- matrix(run(rdist, c(nrng * ntrys, args)), ncol = ntrys)
     apply(rng, 1, extract_valid_sample, lb = lb, ub = ub)
   }
 }
