@@ -120,7 +120,7 @@ stan_predictor.brmsterms <- function(x, data, prior, sparse = FALSE,
   )
   for (nlp in names(x$nlpars)) {
     nlp_args <- list(x$nlpars[[nlp]], center_X = FALSE)
-    out[[nlp]] <- do.call(stan_predictor, c(nlp_args, args))
+    out[[nlp]] <- run(stan_predictor, c(nlp_args, args))
   }
   for (dp in valid_dpars) {
     dp_terms <- x$dpars[[dp]]
@@ -129,7 +129,7 @@ stan_predictor.brmsterms <- function(x, data, prior, sparse = FALSE,
     if (is.btl(dp_terms) || is.btnl(dp_terms)) {
       ilink <- stan_eta_ilink(dp, bterms = x, resp = resp)
       dp_args <- list(dp_terms, ilink = ilink)
-      out[[dp]] <- do.call(stan_predictor, c(dp_args, args))
+      out[[dp]] <- run(stan_predictor, c(dp_args, args))
     } else if (is.numeric(x$fdpars[[dp]]$value)) {
       out[[dp]] <- list(data = dp_def)
     } else if (is.character(x$fdpars[[dp]]$value)) {
@@ -893,32 +893,52 @@ stan_gp <- function(bterms, data, prior, ...) {
   for (i in seq_rows(gpef)) {
     pi <- paste0(p, "_", i)
     byvar <- gpef$byvars[[i]] 
-    bylevels <- gpef$bylevels[[i]]
-    byfac <- length(bylevels) > 0L
+    cons <- gpef$cons[[i]]
+    byfac <- length(cons) > 0L
     bynum <- !is.null(byvar) && !byfac 
+    k <- gpef$k[i]
+    iso <- gpef$iso[i]
     gr <- gpef$gr[i]
+    sfx1 <- gpef$sfx1[[i]]
+    sfx2 <- gpef$sfx2[[i]]
     str_add(out$data) <- paste0(
+      "  // data related to GPs\n",
       "  int<lower=1> Kgp", pi, ";\n",
-      "  int<lower=1> Mgp", pi, ";\n"
+      "  int<lower=1> Dgp", pi, ";\n"
     )
+    if (!isNA(k)) {
+      # !isNA(k) indicates the use of approximate GPs
+      str_add(out$data) <- paste0(
+        "  int<lower=1> NBgp", pi, ";\n"
+      )
+    } 
     str_add(out$par) <- paste0(
       "  // GP hyperparameters\n", 
-      "  vector<lower=0>[Kgp", pi, "] sdgp", pi, ";\n",
-      "  vector<lower=0>[Kgp", pi, "] lscale", pi, ";\n"
-    ) 
+      "  vector<lower=0>[Kgp", pi, "] sdgp", pi, ";\n"
+    )
+    if (gpef$iso[i]) {
+      str_add(out$par) <- paste0(
+        "  vector<lower=0>[1] lscale", pi, "[Kgp", pi, "];\n" 
+      )
+    } else {
+      str_add(out$par) <- paste0(
+        "  vector<lower=0>[Dgp", pi, "] lscale", pi, "[Kgp", pi, "];\n"
+      )
+    }
     str_add(out$prior) <- stan_prior(
-      prior, class = "sdgp", coef = gpef$term[i], 
+      prior, class = "sdgp", coef = sfx1, 
       px = px, suffix = pi
     )
     if (byfac) {
-      J <- seq_along(bylevels)
+      J <- seq_along(cons)
       Ngp <- paste0("Ngp", pi)
       Nsubgp <- paste0("N", if (gr) "sub", "gp", pi)
       Igp <- paste0("Igp", pi, "_", J)
       str_add(out$data) <- paste0(
         "  int<lower=1> ", Ngp, "[Kgp", pi, "];\n",
         collapse(
-          "  int<lower=1> ", Igp, "[", Ngp, "[", J, "]];\n"
+          "  int<lower=1> ", Igp, "[", Ngp, "[", J, "]];\n",
+          "  vector[",  Ngp, "[", J, "]] Cgp", pi, "_", J, ";\n"
         )
       )
       if (gr) {
@@ -929,32 +949,50 @@ stan_gp <- function(bterms, data, prior, ...) {
           )
         )
       }
-      str_add(out$data) <- collapse(
-        "  vector[Mgp", pi, "] Xgp", pi, "_", J, "[", Nsubgp, "[", J, "]];\n"
-      )
-      str_add(out$par) <- collapse(
-        "  vector[", Nsubgp, "[", J, "]] zgp", pi, "_", J, "; \n"
+      gp_call <- paste0("Cgp", pi, "_", J, " .* ")
+      if (!isNA(k)) {
+        str_add(out$data) <- collapse(
+          "  matrix[", Nsubgp, "[", J, "], NBgp", pi, "] Xgp", pi, "_", J, ";\n",
+          "  vector[Dgp", pi, "] slambda", pi, "_", J, "[NBgp", pi, "];\n"
+        )
+        str_add(out$par) <- collapse(
+          "  vector[NBgp", pi, "] zgp", pi, "_", J, "; \n"
+        )
+        str_add(gp_call) <- paste0(
+          "gpa(Xgp", pi, "_", J, ", sdgp", pi, "[", J, "], ", 
+          "lscale", pi, "[", J, "], zgp", pi, "_", J, ", ", 
+          "slambda", pi, "_", J, ")"
+        )
+      } else {
+        str_add(out$data) <- collapse(
+          "  vector[Dgp", pi, "] Xgp", pi, "_", J, "[", Nsubgp, "[", J, "]];\n"
+        )
+        str_add(out$par) <- collapse(
+          "  vector[", Nsubgp, "[", J, "]] zgp", pi, "_", J, "; \n"
+        )
+        str_add(gp_call) <- paste0(
+          "gp(Xgp", pi, "_", J, ", sdgp", pi, "[", J, "], ", 
+          "lscale", pi, "[", J, "], zgp", pi, "_", J, ")"
+        )
+      }
+      Jgp <- if (gr) paste0("[Jgp", pi, "_", J, "]")
+      eta <- combine_prefix(px, keep_mu = TRUE, nlp = TRUE)
+      eta <- paste0(eta, "[", Igp, "]")
+      # compound '+=' statement currently causes a parser failure
+      str_add(out$modelCgp1) <- collapse(
+        "  ", eta, " = ", eta, " + ", gp_call, Jgp, ";\n"
       )
       str_add(out$prior) <- collapse(
         tp(), "normal_lpdf(zgp", pi, "_", J, " | 0, 1);\n"
       )
-      eta <- combine_prefix(px, keep_mu = TRUE, nlp = TRUE)
-      eta <- paste0(eta, "[", Igp, "]")
-      gp_args <- paste0(
-        "Xgp", pi, "_", J, ", sdgp", pi, "[", J, "], ", 
-        "lscale", pi, "[", J, "], zgp", pi, "_", J
-      )
-      Jgp <- if (gr) paste0("[Jgp", pi, "_", J, "]")
-      # compound '+=' statement currently causes a parser failure
-      str_add(out$modelCgp1) <- collapse(
-        "  ", eta, " = ", eta, " + gp(", gp_args, ")", Jgp, ";\n"
-      )
-      coefs <- paste0(gpef$term[i], gpef$bylevels[[i]])
-      str_add(out$prior) <- stan_prior(
-        prior, class = "lscale", coef = coefs, 
-        px = px, suffix = pi
-      )
+      for (j in seq_along(cons)) {
+        str_add(out$prior) <- stan_prior(
+          prior, class = "lscale", coef = sfx2[j, ], 
+          px = px, suffix = paste0(pi, "[", j, "]")
+        )
+      }
     } else {
+      # no by-factor variable
       Nsubgp <- if (gr) paste0("Nsubgp", pi) else "N"
       if (gr) {
         str_add(out$data) <- paste0(
@@ -962,29 +1000,43 @@ stan_gp <- function(bterms, data, prior, ...) {
           "  int<lower=1> Jgp", pi, "[N];\n"
         )
       }
-      str_add(out$data) <- paste0(
-        "  vector[Mgp", pi, "] Xgp", pi, "[", Nsubgp, "];\n"
-      )
+      if (!isNA(k)) {
+        str_add(out$data) <- paste0(
+          "  matrix[", Nsubgp, ", NBgp", pi, "] Xgp", pi, ";\n",
+          "  vector[Dgp", pi, "] slambda", pi, "[NBgp", pi, "];\n"
+        )
+        str_add(out$par) <- paste0(
+          "  vector[NBgp", pi, "] zgp", pi, "; \n"
+        )
+        gp_call <- paste0(
+          "gpa(Xgp", pi, ", sdgp", pi, "[1], lscale", pi, "[1], ", 
+          "zgp", pi, ", slambda", pi, ")"
+        )
+      } else {
+        str_add(out$data) <- paste0(
+          "  vector[Dgp", pi, "] Xgp", pi, "[", Nsubgp, "];\n"
+        ) 
+        str_add(out$par) <- paste0(
+          "  vector[", Nsubgp, "] zgp", pi, "; \n"
+        )
+        gp_call <- paste0(
+          "gp(Xgp", pi, ", sdgp", pi, "[1], lscale", pi, "[1], zgp", pi, ")"
+        )
+      }
       if (bynum) {
         str_add(out$data) <- paste0(
           "  vector[N] Cgp", pi, ";\n"
         )
       }
-      str_add(out$par) <- paste0(
-        "  vector[", Nsubgp, "] zgp", pi, "; \n"
-      )
+      Cgp <- if (bynum) paste0("Cgp", pi, " .* ")
+      Jgp <- if (gr) paste0("[Jgp", pi, "]")
+      str_add(out$eta) <- paste0(" + ", Cgp, gp_call, Jgp)
       str_add(out$prior) <- paste0(
         tp(), "normal_lpdf(zgp", pi, " | 0, 1);\n"
       )
-      gp_args <- paste0(
-        "Xgp", pi, ", sdgp", pi, "[1], lscale", pi, "[1], zgp", pi
-      )
-      Cgp <- if (bynum) paste0("Cgp", pi, " .* ")
-      Jgp <- if (gr) paste0("[Jgp", pi, "]")
-      str_add(out$eta) <- paste0(" + ", Cgp, "gp(", gp_args, ")", Jgp)
       str_add(out$prior) <- stan_prior(
-        prior, class = "lscale", coef = gpef$term[i], 
-        px = px, suffix = pi
+        prior, class = "lscale", coef = sfx2, 
+        px = px, suffix = paste0(pi, "[1]")
       )
     }
   }
