@@ -1,5 +1,5 @@
 params <-
-list(EVAL = TRUE)
+list(EVAL = FALSE)
 
 ## ---- SETTINGS-knitr, include=FALSE-----------------------------------------------------
 stopifnot(require(knitr))
@@ -20,34 +20,34 @@ library(brms)
 theme_set(theme_default())
 
 ## ---------------------------------------------------------------------------------------
-data("nhanes", package = "mice")
-head(nhanes)
+#  data("nhanes", package = "mice")
+#  head(nhanes)
 
 ## ---------------------------------------------------------------------------------------
-library(mice)
-imp <- mice(nhanes, m = 5, print = FALSE)
+#  library(mice)
+#  imp <- mice(nhanes, m = 5, print = FALSE)
 
 ## ---- results = 'hide', message = FALSE-------------------------------------------------
-fit_imp1 <- brm_multiple(bmi ~ age*chl, data = imp, chains = 2)
+#  fit_imp1 <- brm_multiple(bmi ~ age*chl, data = imp, chains = 2)
 
 ## ---------------------------------------------------------------------------------------
-summary(fit_imp1)
+#  summary(fit_imp1)
 
 ## ---------------------------------------------------------------------------------------
-plot(fit_imp1, pars = "^b")
+#  plot(fit_imp1, pars = "^b")
 
 ## ---------------------------------------------------------------------------------------
-round(fit_imp1$rhats, 2)
+#  round(fit_imp1$rhats, 2)
 
 ## ---------------------------------------------------------------------------------------
-marginal_effects(fit_imp1, "age:chl")
+#  marginal_effects(fit_imp1, "age:chl")
 
 ## ---- results = 'hide', message = FALSE-------------------------------------------------
-bform <- bf(bmi | mi() ~ age * mi(chl)) +
-  bf(chl | mi() ~ age) + set_rescor(FALSE)
-fit_imp2 <- brm(bform, data = nhanes)
+#  bform <- bf(bmi | mi() ~ age * mi(chl)) +
+#    bf(chl | mi() ~ age) + set_rescor(FALSE)
+#  fit_imp2 <- brm(bform, data = nhanes)
 
 ## ---------------------------------------------------------------------------------------
-summary(fit_imp2)
-marginal_effects(fit_imp2, "age:chl", resp = "bmi")
+#  summary(fit_imp2)
+#  marginal_effects(fit_imp2, "age:chl", resp = "bmi")
 
